@@ -20,10 +20,39 @@ extern struct Nvaudio_card *devs;
 extern unsigned int spdif_status;
 
 
-/* in this loop, dmabuf.count signifies the amount of data that is
-   waiting to be copied to the user's buffer.  It is filled by the dma
-   machine and drained by this loop. */
+/**********************************************************************
+*  Funtion to verify for ac3 data.
+*  This check the first few data for ac3 signature. 
+***********************************************************************/
+static int  Nvaudio_verifyac3(struct Nvaudio_card *card,const char *buffer, size_t count)
+{
+    unsigned short ac3data[0x40];
+    int ac3size = 0, i = 0;
+    ac3size = sizeof(short) * 0x40;
+    memset(ac3data, 0, ac3size);
+    
+    if(count < ac3size) return 0;
+    copy_from_user(ac3data,buffer,ac3size);
 
+    for( i=0; i < 0x40; i++) {
+        if(ac3data[i] == 0xF872) {
+            i++;
+            if(ac3data[i] == 0x4E1F) {
+                card->waveformat     = AFMT_AC3;
+                card->ac3passthrough = 1;
+                return 1;
+            }
+        }
+    }
+    return 0;
+}
+
+/**********************************************************************
+*   Nvaudio_read which helps to pass the recorded data from hw to user.
+*   in this loop, dmabuf.count signifies the amount of data that is
+*   waiting to be copied to the user's buffer.  It is filled by the dma
+*   machine and drained by this loop. 
+**********************************************************************/
 static ssize_t Nvaudio_read(struct file *file, char *buffer, size_t count, loff_t *ppos)
 {
     ssize_t ret;
@@ -177,8 +206,12 @@ static ssize_t Nvaudio_read(struct file *file, char *buffer, size_t count, loff_
     return ret;
 }
 
-/* in this loop, dmabuf.count signifies the amount of data that is waiting to be dma to
-   the soundcard.  it is drained by the dma machine and filled by this loop. */
+/*******************************************************************************
+*  Function that passed the data from user to hw for playback.   
+*  in this loop, dmabuf.count signifies the amount of data that is waiting
+*  to be dma to the soundcard.  
+*  it is drained by the dma machine and filled by this loop. 
+******************************************************************************/
 static ssize_t Nvaudio_write(struct file *file, const char *buffer, size_t count, loff_t *ppos)
 {
     ssize_t ret = 0;
@@ -189,21 +222,31 @@ static ssize_t Nvaudio_write(struct file *file, const char *buffer, size_t count
 #endif
     unsigned long flags;
     unsigned int swptr = 0;
-    int cnt, x;
+    int cnt = 0, x = 0, newcnt = 0, failcount = 0;
+    u32 samplecount    = 0;
     struct Nvaudio_card *card   = (struct Nvaudio_card *)file->private_data;
     struct Nvaudio_state *state = card ? card->wavout: 0;
     struct dmabuf *dmabuf = &state->dmabuffer;
 
     DECLARE_WAITQUEUE(waita, current);
     /*check whether it is digital only - no analog play */
-    if(card->waveformat == AFMT_AC3) {
+    if((card->waveformat == AFMT_AC3) || (!card->analogstate)) {
         goto ac3only;
     }
 
+    /* ac3 data can send with  16 bit format*/
+    if(card->verifyac3 == 0) {
+        card->verifyac3 =1;
+        if(Nvaudio_verifyac3(card,buffer,count)){
+            goto ac3only;
+        }
+    }
+
 #ifdef NV_DEBUG_PLAY
     printk("Nvaudio_write called, count = %d\n", count);
 #endif
     if(!state) return -EFAULT;
+
     if (ppos != &file->f_pos)
         return -ESPIPE;
     if (state->mapped)
@@ -216,10 +259,28 @@ static ssize_t Nvaudio_write(struct file *file, const char *buffer, size_t count
 
     ret = 0;
 
+    if(count == 0) return ret;
+    count &= ~0x3;
+    newcnt = count;
+
+    /* Do src and copy the data to linked list */
+    if((count > 0) && (state->rate != SAMPLERATE_48K)) {
+        if(NULL == Nvaudio_linearSRC(card,buffer,count)) {
+            return ret;
+        }
+    }
+
     add_wait_queue(&dmabuf->wait, &waita);
 
-    while (count > 0) {
+    while( (newcnt > 0) || (card->outList->wavoutcount)) {
+
         set_current_state(TASK_INTERRUPTIBLE);
+        
+        if(!card->analogstate)  {
+           if (!ret) ret = -EAGAIN;
+           goto writedone;
+        }
+
         spin_lock_irqsave(&state->lock, flags);
         if (PM_SUSPENDED(card)) {
             spin_unlock_irqrestore(&state->lock, flags);
@@ -246,8 +307,12 @@ static ssize_t Nvaudio_write(struct file *file, const char *buffer, size_t count
 #ifdef NV_DEBUG_PLAY
         printk(KERN_INFO " Nvaudio_write: %d bytes available space\n", cnt);
 #endif
-        if (cnt > count)
-            cnt = count;
+        if(state->rate == SAMPLERATE_48K) {
+            /*check whether the calc is right or not */
+            cnt = Nvaudio_Recalfreespace(card,cnt);
+            if (cnt > newcnt)
+                cnt = newcnt;
+        }
         /* Lop off the last two bits to force the code to always
          * write in full samples.  This keeps software that sets
          * O_NONBLOCK but doesn't check the return value of the
@@ -267,8 +332,9 @@ static ssize_t Nvaudio_write(struct file *file, const char *buffer, size_t count
             state->trigger = PCM_ENABLE_OUTPUT;
             Nvaudio_update_lvi(state);
 
-            if (file->f_flags & O_NONBLOCK) {
+            if ((file->f_flags & O_NONBLOCK) && (state->rate == SAMPLERATE_48K)) {
                 if (!ret) ret = -EAGAIN;
+                failcount = 1;
                 goto writedone;
             }
             /* Not strictly correct but works */
@@ -291,10 +357,10 @@ static ssize_t Nvaudio_write(struct file *file, const char *buffer, size_t count
 #endif
                 /* a buffer underrun, we delay the recovery until next time the
                    while loop begin and we REALLY have data to play */
-                //return ret;
             }
             if (signal_pending(current)) {
                 if (!ret) ret = -ERESTARTSYS;
+                failcount = 1;
                 goto writedone;
             }
             continue;
@@ -305,27 +371,33 @@ static ssize_t Nvaudio_write(struct file *file, const char *buffer, size_t count
            buffer,cnt,dmabuf->rawbuf + swptr);
 #endif
 
-        if (copy_from_user(dmabuf->rawbuf+swptr,buffer,cnt)) {
-            if (!ret) ret = -EFAULT;
-            goto writedone;
-        }
+        if(state->rate == SAMPLERATE_48K) {
+            samplecount = Nvaudio_wavoutwrite(card, buffer, (void *)(dmabuf->rawbuf+swptr), cnt);
 
-        swptr = (swptr + cnt) % dmabuf->dmasize;
+            swptr  = (swptr + samplecount) % dmabuf->dmasize;
 
-        spin_lock_irqsave(&state->lock, flags);
-        if (PM_SUSPENDED(card)) {
+            spin_lock_irqsave(&state->lock, flags);
+            if (PM_SUSPENDED(card)) {
                 spin_unlock_irqrestore(&state->lock, flags);
                 continue;
+            }
+
+            dmabuf->swptr = swptr;
+            dmabuf->count += samplecount;
+            spin_unlock_irqrestore(&state->lock, flags);
+
+            newcnt  -= cnt;
+            buffer += cnt;
+            ret    += cnt;
+        }else {
+            newcnt = count;
+            if(cnt < newcnt){
+                newcnt = cnt;
+            }
+            newcnt = Nvaudio_copydatatowavprd(card,newcnt);
+            swptr = dmabuf->swptr;
         }
 
-        dmabuf->swptr = swptr;
-        dmabuf->count += cnt;
-        spin_unlock_irqrestore(&state->lock, flags);
-
-        count  -= cnt;
-        buffer += cnt;
-        ret    += cnt;
-
 #ifdef NV_DEBUG_PLAY
         printk("Play swptr %x dmacount %x \n", swptr,dmabuf->count);
 #endif
@@ -340,17 +412,30 @@ writedone:
     set_current_state(TASK_RUNNING);
     remove_wait_queue(&dmabuf->wait, &waita);
 
+    if(!failcount) {
 ac3only:
 #ifdef ENABLE_SPDIF
-    if((state->spdif_enable)&&(card->spout)){
-         spret = Nvaudio_spdifwrite(card,spbuffer,spcount);
-         if(card->waveformat == AFMT_AC3) ret = spret;
-    }
+        if((state->spdif_enable)&&(card->spout)){
+             spret = Nvaudio_spdifwrite(card,spbuffer,spcount);
+             if(card->waveformat == AFMT_AC3) ret = spret;
+             else if((card->digitalstate) && !(card->analogstate)) ret = count;
+        }else {
+            if(card->waveformat == AFMT_AC3) ret = count;
+        }
 #endif
+
+        if(state->rate != 48000){
+            buffer += count;
+            ret    += count;
+        }
+    }
     return ret;
 }
 
-/* No kernel lock - we have our own spinlock */
+/**********************************************************************
+* Function to poll the hw status
+* No kernel lock - we have our own spinlock 
+**********************************************************************/
 static unsigned int Nvaudio_poll(struct file *file, struct poll_table_struct *wait)
 {
     struct dmabuf *dmabuf       = NULL;
@@ -363,7 +448,11 @@ static unsigned int Nvaudio_poll(struct file *file, struct poll_table_struct *wa
     printk("Nvaudio: Nvaudio_poll ");
 #endif
     if(file->f_mode & FMODE_WRITE)  {
-        poll_wait(file, &card->wavout->dmabuffer.wait, wait);
+        if(card->spout && (!card->analogstate && card->digitalstate)) {
+            poll_wait(file, &card->spout->dmabuffer.wait, wait);  
+        } else {
+            poll_wait(file, &card->wavout->dmabuffer.wait, wait);   
+        }
     }
 
     if(file->f_mode & FMODE_READ)
@@ -384,8 +473,13 @@ static unsigned int Nvaudio_poll(struct file *file, struct poll_table_struct *wa
     }
 
     if(file->f_mode & FMODE_WRITE){
-       state  = card->wavout;
-       dmabuf = &state->dmabuffer;
+        if(card->spout && (!card->analogstate && card->digitalstate)) {
+            state  = card->spout;
+            dmabuf = &state->dmabuffer;
+        } else  {
+            state  = card->wavout;
+            dmabuf = &state->dmabuffer;
+        }
        if(!state->ready)     return 0;
        spin_lock_irqsave(&state->lock, flags);
        if (state->enable & DAC_RUNNING ||
@@ -401,6 +495,10 @@ static unsigned int Nvaudio_poll(struct file *file, struct poll_table_struct *wa
 #endif
     return mask;
 }
+
+/**********************************************************************
+* Function to enable the mmap settings
+**********************************************************************/
 static int Nvaudio_mmapwrite(struct vm_area_struct *vma,struct Nvaudio_state *state,unsigned rec)
 {
     int ret = -EINVAL;
@@ -440,6 +538,9 @@ static int Nvaudio_mmapwrite(struct vm_area_struct *vma,struct Nvaudio_state *st
 
 }
 
+/**********************************************************************
+* Function to do mmap ioctls
+**********************************************************************/
 static int Nvaudio_mmap(struct file *file, struct vm_area_struct *vma)
 {
     struct Nvaudio_state *dacstate = NULL;
@@ -461,22 +562,24 @@ static int Nvaudio_mmap(struct file *file, struct vm_area_struct *vma)
 
         ret = -EINVAL;
 
-        if (vma->vm_pgoff != 0)
-          goto out;
+        if(!(vma->vm_flags & VM_WRITE)) {
+            if (vma->vm_pgoff != 0)
+              goto out;
 
-        size = vma->vm_end - vma->vm_start;
-        if (size > (PAGE_SIZE << recdmabuf->buforder))
-          goto out;
-        ret = -EAGAIN;
+            size = vma->vm_end - vma->vm_start;
+            if (size > (PAGE_SIZE << recdmabuf->buforder))
+              goto out;
+            ret = -EAGAIN;
 
 #if defined(REMAP_NEW)
-        if (remap_page_range(vma, vma->vm_start, virt_to_phys(recdmabuf->rawbuf),
+            if (remap_page_range(vma, vma->vm_start, virt_to_phys(recdmabuf->rawbuf),
                  size, vma->vm_page_prot))
 #else
-        if (remap_page_range(vma->vm_start, virt_to_phys(recdmabuf->rawbuf),
+            if (remap_page_range(vma->vm_start, virt_to_phys(recdmabuf->rawbuf),
                  size, vma->vm_page_prot))
 #endif
-           goto out;
+            goto out;
+        }
         recstate->mapped = 1;
         recstate->trigger = 0;
         ret = 0;
@@ -484,14 +587,20 @@ static int Nvaudio_mmap(struct file *file, struct vm_area_struct *vma)
     if(vma->vm_flags & VM_WRITE){
        dacstate = card->wavout;
 #ifdef ENABLE_SPDIF
-       if((dacstate->spdif_enable) && (card->spout)) {
+       /*if((dacstate->spdif_enable) && (card->spout)) {
           if( (ret =  Nvaudio_mmapwrite(vma,card->spout,2)) != 0)
               goto out;
-       }
+       }*/
        /* Disable the Spdif right now */
        dacstate->spdif_enable = 0;
 #endif
-         ret = Nvaudio_mmapwrite(vma,card->wavout,0);
+       ret = Nvaudio_mmapwrite(vma,card->wavout,0);
+       /* set the samplerate variable now as we donot support spdif with mmap */
+       Nvaudio_set_dac_rate(dacstate,dacstate->rate);
+       /* set the speaker to stereo mode now as we donot support
+       software SRC & downmix to remapped data */
+       Nvaudio_SetSpkrRegister(card);
+
     }
 #ifdef NV_DEBUG_NORMAL
     printk("Nvaudio: mmap'ed %ld bytes of data space\n", size);
@@ -501,6 +610,11 @@ out:
     return ret;
 }
 
+/**********************************************************************
+*  Function that supports the fragment size for hw and user buffer
+*  Avoiding smaller buffer as it glitches with the games.
+*  Need to verify whether it will create problem with other apps
+**********************************************************************/
 int Nvaudio_setfragment(struct dmabuf* dmabuf, int val)
 {
    dmabuf->ossfragsize = 1 << (val & 0xffff);
@@ -517,11 +631,11 @@ int Nvaudio_setfragment(struct dmabuf* dmabuf, int val)
           dmabuf->ossfragsize, dmabuf->ossmaxfrags);
 #endif
 
-     if (dmabuf->ossfragsize < 256)
-       dmabuf->ossfragsize = 256;
+     //if (dmabuf->ossfragsize < 256)
+      // dmabuf->ossfragsize = 256;
      //small buffersize giving glitch - so setting to high value
-     //if (dmabuf->ossfragsize < 1024)
-     //   dmabuf->ossfragsize = 1024;
+     if (dmabuf->ossfragsize < 1024)
+        dmabuf->ossfragsize = 1024;
      else if (dmabuf->ossfragsize > 4096)
           dmabuf->ossfragsize = 4096;
       /*
@@ -542,17 +656,16 @@ int Nvaudio_setfragment(struct dmabuf* dmabuf, int val)
          * be 2.
          */
 
-        if(dmabuf->ossmaxfrags > 256)
+        /*if(dmabuf->ossmaxfrags > 256)
             dmabuf->ossmaxfrags = 256;
         else if (dmabuf->ossmaxfrags < 2)
-           dmabuf->ossmaxfrags = 2;
+           dmabuf->ossmaxfrags = 2;*/
         //small fragsize giving glitch - so setting to high value
-        /*if (dmabuf->ossmaxfrags > 64)
+        if (dmabuf->ossmaxfrags > 64)
            dmabuf->ossmaxfrags = 64;
         else if (dmabuf->ossmaxfrags < 16)
            dmabuf->ossmaxfrags = 16;
-    */
-
+    
         val = dmabuf->ossfragsize * dmabuf->ossmaxfrags;
 
         while (val < 8192) {
@@ -572,6 +685,9 @@ int Nvaudio_setfragment(struct dmabuf* dmabuf, int val)
         return val;
 }
 
+/**********************************************************************
+* Nvaudio_dsppost
+**********************************************************************/
 static void Nvaudio_dsppost(struct Nvaudio_state *state)
 {
     unsigned long  flags;
@@ -587,6 +703,9 @@ static void Nvaudio_dsppost(struct Nvaudio_state *state)
 
 }
 
+/**********************************************************************
+* Nvaudio_dsp_reset
+**********************************************************************/
 void Nvaudio_dsp_reset(struct Nvaudio_state *state,unsigned rec)
 {
     unsigned long  flags;
@@ -631,7 +750,10 @@ void Nvaudio_dsp_reset(struct Nvaudio_state *state,unsigned rec)
     dmabuf->count = dmabuf->total_bytes = 0;
 
 }
-/* All the OSS ioctls are supported in this function */
+
+/**********************************************************************
+* All the OSS ioctls are supported in this function
+**********************************************************************/
 static int Nvaudio_ioctl(struct inode *inode, struct file *file, unsigned int cmd, unsigned long arg)
 {
     unsigned long  flags;
@@ -711,7 +833,7 @@ static int Nvaudio_ioctl(struct inode *inode, struct file *file, unsigned int cm
                     return val;
 
                 dacdmabuf->total_bytes = 0;
-                spdmabuf->total_bytes = 0;
+                spdmabuf->total_bytes  = 0;
             }
             return 0;
 
@@ -871,8 +993,9 @@ static int Nvaudio_ioctl(struct inode *inode, struct file *file, unsigned int cm
                     /*Set the channels getting used; */
                     card->numchannels = val;
 
-                    outl(i_glob_cnt & 0xcfffff,
-                         card->iobase + GLOB_CNT);
+                    /* Check the Spkr configuration to set the Register*/
+                    Nvaudio_SetSpkrRegister(card);
+
                     /* Do we need to change mixer settings????  */
                     break;
                 case 4: /* Supported on some chipsets, better check first */
@@ -881,11 +1004,11 @@ static int Nvaudio_ioctl(struct inode *inode, struct file *file, unsigned int cm
                         /*Set the channels getting used; */
                         card->numchannels = val;
 
-                        outl((i_glob_cnt & 0xcfffff) | 0x100000,
-                              card->iobase + GLOB_CNT);
+                        /* Check the Spkr configuration to set the Register*/
+                        Nvaudio_SetSpkrRegister(card);
 
                         /* Read the Register and set as well*/
-                        Nvaudio_MultiChannel_volume(card,val);
+                        //Nvaudio_MultiChannel_volume(card,val);
                         
 #ifdef NV_DEBUG_NORMAL
                         printk("pcm 0x%x, master 0x%x, sur 0x%x, reg64 0x%x  \n",
@@ -905,10 +1028,11 @@ static int Nvaudio_ioctl(struct inode *inode, struct file *file, unsigned int cm
                         /*Set the channels getting used; */
                         card->numchannels = val;
 
-                        outl((i_glob_cnt & 0xcfffff) | 0x200000,
-                              card->iobase + GLOB_CNT);
+                        /* Check the Spkr configuration to set the Register*/
+                        Nvaudio_SetSpkrRegister(card);
+
                         /* Read the Register and set as well*/
-                        Nvaudio_MultiChannel_volume(card,val);
+                        //Nvaudio_MultiChannel_volume(card,val);
                         
 #ifdef NV_DEBUG_NORMAL
                              printk(" channel 6 pcm 0x%x, master 0x%x, sur 0x%x , center 0x%x, reg64 0x%x , reg66 0x%x \n",
@@ -1017,15 +1141,7 @@ static int Nvaudio_ioctl(struct inode *inode, struct file *file, unsigned int cm
                 return -EINVAL;
             if (!dacstate->ready && (val = prog_dmabuf(dacstate, 0)) != 0)
                 return val;
-
-#ifdef ENABLE_SPDIF
-            if((dacstate->spdif_enable)&&(spstate)) {
-                if (!spstate->ready && (val = prog_dmabuf(spstate, 2)) != 0){
-                     /* Do we need to return - no */
-                }
-            }
-#endif
-
+            
             spin_lock_irqsave(&dacstate->lock, flags);
             Nvaudio_update_ptr(dacstate);
             abinfo.fragsize     = dacdmabuf->userfragsize;
@@ -1036,6 +1152,28 @@ static int Nvaudio_ioctl(struct inode *inode, struct file *file, unsigned int cm
                 abinfo.bytes    = Nvaudio_get_free_write_space(dacstate);
             abinfo.fragments    = abinfo.bytes / dacdmabuf->userfragsize;
             spin_unlock_irqrestore(&dacstate->lock, flags);
+
+#ifdef ENABLE_SPDIF
+            if((dacstate->spdif_enable)&&(spstate)) {
+                if (!spstate->ready && (val = prog_dmabuf(spstate, 2)) != 0){
+                     /* Do we need to return - no */
+                }
+                if(!card->analogstate && card->digitalstate)
+                {
+                    spin_lock_irqsave(&spstate->lock, flags);
+                    Nvaudio_update_ptr(spstate);
+                    abinfo.fragsize     = spdmabuf->userfragsize;
+                    abinfo.fragstotal   = spdmabuf->userfrags;
+                    if (dacstate->mapped)
+                        abinfo.bytes    = spdmabuf->dmasize;
+                    else
+                        abinfo.bytes    = Nvaudio_get_free_write_space(spstate);
+                    abinfo.fragments    = abinfo.bytes / spdmabuf->userfragsize;
+                    spin_unlock_irqrestore(&spstate->lock, flags);
+
+                }
+            }
+#endif
 #if defined(NV_DEBUG_NORMAL) || defined(DEBUG_MMAP)
             printk("SNDCTL_DSP_GETOSPACE %d, %d, %d, %d\n", abinfo.bytes,
                  abinfo.fragsize, abinfo.fragments, abinfo.fragstotal);
@@ -1130,8 +1268,8 @@ static int Nvaudio_ioctl(struct inode *inode, struct file *file, unsigned int cm
 #ifdef NV_DEBUG_NORMAL
             printk("SNDCTL_DSP_GETCAPS\n");
 #endif
-            return put_user(DSP_CAP_REALTIME|DSP_CAP_TRIGGER|DSP_CAP_MMAP|DSP_CAP_BIND,
-                    (int *)arg);
+            return put_user(DSP_CAP_DUPLEX | DSP_CAP_REALTIME | DSP_CAP_TRIGGER|
+                            DSP_CAP_MMAP | DSP_CAP_BIND | DSP_CAP_MULTI,(int *)arg);
 
         case SNDCTL_DSP_GETTRIGGER:
             val = 0;
@@ -1272,6 +1410,8 @@ static int Nvaudio_ioctl(struct inode *inode, struct file *file, unsigned int cm
 #endif
             return put_user(card->waveformat, (int *)arg);
 
+#if LINUX_VERSION_CODE >= KERNEL_VERSION(2,4,9)
+
         case SNDCTL_DSP_SETSPDIF: /* Set S/PDIF Control register */
 #ifdef NV_DEBUG_NORMAL
             printk("SNDCTL_DSP_SETSPDIF\n");
@@ -1320,6 +1460,7 @@ static int Nvaudio_ioctl(struct inode *inode, struct file *file, unsigned int cm
             }
             //return put_user((val & 0xcfff), (int *)arg);
             return put_user(val, (int *)arg);
+#endif
 
         case SNDCTL_DSP_GETCHANNELMASK:
 #ifdef NV_DEBUG_NORMAL
@@ -1430,6 +1571,9 @@ static int Nvaudio_ioctl(struct inode *inode, struct file *file, unsigned int cm
         return -EINVAL;
 }
 
+/**********************************************************************
+* Function to support the open calls
+**********************************************************************/
 static int Nvaudio_open(struct inode *inode, struct file *file)
 {
     int i  = 0;
@@ -1451,6 +1595,7 @@ static int Nvaudio_open(struct inode *inode, struct file *file)
             set_current_state(TASK_UNINTERRUPTIBLE);
             schedule_timeout(HZ/20);
         }
+
         if(card && !card->initializing ){
 #ifdef NV_DEBUG_NORMAL
             printk("Nvaudio_open without card init \n");
@@ -1472,6 +1617,7 @@ static int Nvaudio_open(struct inode *inode, struct file *file)
                 state  = card->wavin;
                 memset(state, 0, sizeof(struct Nvaudio_state));
                 dmabuf = &state->dmabuffer;
+                spin_lock_init(&state->lock);
 
                 state->card = card;
                 init_waitqueue_head(&dmabuf->wait);
@@ -1525,6 +1671,7 @@ static int Nvaudio_open(struct inode *inode, struct file *file)
                 state  = card->wavout;
                 memset(state, 0, sizeof(struct Nvaudio_state));
                 dmabuf = &state->dmabuffer;
+                spin_lock_init(&state->lock);
 
                 state->card  = card;
                 init_waitqueue_head(&dmabuf->wait);
@@ -1553,6 +1700,7 @@ static int Nvaudio_open(struct inode *inode, struct file *file)
                 state->open_mode |= file->f_mode & (FMODE_WRITE);
                 spin_lock_init(&state->lock);
                 state->spdif_enable = 0;
+                card->verifyac3     = 0;
 
 #ifdef ENABLE_SPDIF
                 if(spdif_status) {
@@ -1573,6 +1721,9 @@ static int Nvaudio_open(struct inode *inode, struct file *file)
     return 0;
 }
 
+/**********************************************************************
+* Function to support the release/close calls
+**********************************************************************/
 static int Nvaudio_release(struct inode *inode, struct file *file)
 {
     struct Nvaudio_state *state    = NULL;
@@ -1591,6 +1742,7 @@ static int Nvaudio_release(struct inode *inode, struct file *file)
         if(state->trigger & PCM_ENABLE_INPUT) {
                stop_adc(state);
         }
+
         spin_lock_irqsave(&state->lock, flags);
         dealloc_dmabuf(state);
         spin_unlock_irqrestore(&state->lock, flags);
@@ -1611,6 +1763,7 @@ static int Nvaudio_release(struct inode *inode, struct file *file)
 #ifdef ENABLE_SPDIF
         if(card->spout)
             dacstate->spdif_enable = 1;
+
         if(dacstate->spdif_enable) {
            state  = card->spout;
            dmabuf = &state->dmabuffer;
@@ -1624,16 +1777,28 @@ static int Nvaudio_release(struct inode *inode, struct file *file)
            card->spout = NULL;
        }
 #endif
+        dacstate->mapped = 0;
+        /* set the speaker to existing settings */
+        Nvaudio_SetSpkrRegister(card);
         kfree(dacstate);
         card->wavout = NULL;
+
+        if(card->outList) {
+            Nvaudio_resetnode(card);
+        }
     }
+    card->ac3passthrough = 0;
+    card->waveformat     = AFMT_S16_LE;
+    card->verifyac3      = 0;
     unlock_kernel();
     return 0;
 }
 
 struct file_operations Nvaudio_fops = {
     owner:      THIS_MODULE,
+#if LINUX_VERSION_CODE >= KERNEL_VERSION(2,4,9)
     llseek:     no_llseek,
+#endif
     read:       Nvaudio_read,
     write:      Nvaudio_write,
     poll:       Nvaudio_poll,
