@@ -12,7 +12,6 @@
 #include "nvwavout.h"
 #include "nvspdif.h"
 
-extern int strict_clocking;
 extern unsigned int clocking;
 extern int mastervolright;
 extern int mastervolleft;
@@ -22,45 +21,25 @@ extern int pcmvolleft;
 static short SPDIFLeftovers[6];
 static u32 SPDIFLeftoverCount = 0;
 
-/* set spdifout sample rate */
+/**********************************************************************
+* set spdifout sample rate 
+**********************************************************************/
 unsigned int Nvaudio_set_spdif_rate(struct Nvaudio_state *state, unsigned int rate)
 {
-    u32 new_rate;
-    struct ac97_codec *codec=state->card->ac97_codec[0];
-
-    if(!(state->card->ac97_features&0x0001))  {
-        state->rate = clocking;
-        return clocking;
-    }
-
     if (rate > 48000)
         rate = 48000;
     if (rate < 8000)
         rate = 8000;
+    
+    rate = ( rate * clocking)/clocking;
     state->rate = rate;
-
-    /*
-     *  Adjust for misclocked crap
-     */
-    rate = ( rate * clocking)/48000;
-    if(strict_clocking && rate < 8000) {
-        rate = 8000;
-        state->rate = (rate * 48000)/clocking;
-    }
-
-    new_rate=ac97_set_dac_rate(codec, rate);
-    if(new_rate != rate) {
-        state->rate = (new_rate * 48000)/clocking;
-    }
-#ifdef NV_DEBUG_SP
-    printk("Nvaudio_set_spdif_rate : asked for %d, got %d\n", rate, state->rate);
-#endif
-    rate = new_rate;
     return state->rate;
 }
 
-/* get current playbackdma buffer pointer (byte offset from LBA),
-   called with spinlock held! */
+/**********************************************************************
+* get current playbackdma buffer pointer (byte offset from LBA),
+* called with spinlock held!
+**********************************************************************/
 unsigned Nvaudio_get_spdma_addr(struct Nvaudio_state *state)
 {
     unsigned int civ, offset, port, port_picb, bytes = 2;
@@ -90,7 +69,9 @@ unsigned Nvaudio_get_spdma_addr(struct Nvaudio_state *state)
         % dmabuf->dmasize);
 }
 
-/* stop playback (lock held) */
+/**********************************************************************
+* stop playback (lock held)
+**********************************************************************/
 void __stop_spdif(struct Nvaudio_state *state)
 {
     struct Nvaudio_card *card = state->card;
@@ -113,10 +94,13 @@ void stop_spdif(struct Nvaudio_state *state)
     spin_unlock_irqrestore(&state->lock, flags);
 }
 
+/**********************************************************************
+* start spdif out
+**********************************************************************/
 void __start_spdif(struct Nvaudio_state *state)
 {
     struct dmabuf *dmabuf = &state->dmabuffer;
-    if (dmabuf->count > 0 && state->ready && !state->enable &&
+    if (state->card->digitalstate && dmabuf->count > 0 && state->ready && !state->enable &&
         (state->trigger & PCM_ENABLE_OUTPUT)) {
          state->enable = SPDIF_RUNNING;
 
@@ -136,6 +120,10 @@ void start_spdif(struct Nvaudio_state *state)
     spin_unlock_irqrestore(&state->lock, flags);
 }
 
+/**********************************************************************
+* update lvi 
+**********************************************************************/
+
 void __Nvaudio_update_splvi(struct Nvaudio_state *state)
 {
     int x, port = 0;
@@ -145,8 +133,10 @@ void __Nvaudio_update_splvi(struct Nvaudio_state *state)
         if (dmabuf->count &&
            (state->trigger & PCM_ENABLE_OUTPUT)) {
             outb((inb(port+SP_CIV)+1)&31, port+SP_LVI);
-            __start_spdif(state);
-            while( (!(inb(port + SP_CR) & (0x10|0x04)))) ;
+            if(state->card->digitalstate)  {
+                __start_spdif(state);
+                while( (!(inb(port + SP_CR) & (0x10|0x04)))) ;
+            }
         }
     }
 
@@ -168,7 +158,10 @@ void Nvaudio_update_splvi(struct Nvaudio_state *state)
     spin_unlock_irqrestore(&state->lock, flags);
 }
 
-/* update buffer manangement pointers, especially, dmabuf->count and dmabuf->hwptr */
+/**********************************************************************
+* update buffer manangement pointers, 
+* especially, dmabuf->count and dmabuf->hwptr
+**********************************************************************/
 void Nvaudio_update_spptr(struct Nvaudio_state *state)
 {
     unsigned hwptr = 0;
@@ -214,7 +207,9 @@ void Nvaudio_update_spptr(struct Nvaudio_state *state)
 
 }
 
-/* get the count of free space in the prds */
+/**********************************************************************
+* get the count of free space in the prds
+**********************************************************************/
 int Nvaudio_get_free_spwrite_space(struct Nvaudio_state *state)
 {
     int free;
@@ -240,8 +235,9 @@ int Nvaudio_get_free_spwrite_space(struct Nvaudio_state *state)
         return(0);
     return(free);
 }
-
-/* function to drain th existing data from spdif*/
+/**********************************************************************
+* function to drain th existing data from spdif
+**********************************************************************/
 int drain_spdif(struct Nvaudio_state *state, int signals_allowed)
 {
     unsigned long flags;
@@ -302,6 +298,9 @@ int drain_spdif(struct Nvaudio_state *state, int signals_allowed)
     return 0;
 }
 
+/**********************************************************************
+* spdif open
+**********************************************************************/
 int Nvaudio_spdifopen(struct Nvaudio_card  *card)
 {
     int i =0;
@@ -317,6 +316,7 @@ int Nvaudio_spdifopen(struct Nvaudio_card  *card)
     memset(state, 0, sizeof(struct Nvaudio_state));
     dmabuf = &state->dmabuffer;
     state->card  = card;
+    spin_lock_init(&state->lock);
     init_waitqueue_head(&dmabuf->wait);
 
     state->trigger = 0;
@@ -338,21 +338,26 @@ int Nvaudio_spdifopen(struct Nvaudio_card  *card)
     return 0;
 }
 
-/* copy the data from the UserBuffer to Prd Dma */
+/**********************************************************************
+* copy the data from the UserBuffer to Prd Dma 
+**********************************************************************/
 ssize_t Nvaudio_spdifwrite(struct Nvaudio_card *card, const char *buffer, size_t count)
 {
     ssize_t ret;
     unsigned long flags;
     unsigned int swptr = 0;
-    int cnt, x, loop, loopend,shiftdone = 0;
-    u32 SampleCount = 0, StartIndex = 0;
-    short *pDest = NULL, *pSource = NULL; 
-    long left=0,right=0,center=0,lfe=0,sleft=0,sright=0;
+    int cnt, x, loopend = 0, newcnt = 0;
+    u32 SampleCount = 0;
     struct Nvaudio_state *state = card->spout;
     struct dmabuf *dmabuf = &state->dmabuffer;
 
     DECLARE_WAITQUEUE(waits, current);
 
+    if(!card->digitalstate) {
+        ret = count;
+        return ret;
+    }
+
 #ifdef NV_DEBUG_SP
     printk("Nvaudio_spwrite called, count = %d\n", count);
 #endif
@@ -366,9 +371,29 @@ ssize_t Nvaudio_spdifwrite(struct Nvaudio_card *card, const char *buffer, size_t
 
     ret = 0;
 
+    if((state->rate == SAMPLERATE_48K)|| (card->ac3passthrough))
+        newcnt = count;
+    else
+        newcnt = 0;
+
+    if((!card->analogstate) &&
+        (count > 0) && (state->rate != 48000)) {
+        if(NULL == Nvaudio_linearSRC(card,buffer,count)) {
+            printk(KERN_ERR " More space for SRC \n");
+            return -EFAULT;
+        }
+    
+    }
     add_wait_queue(&dmabuf->wait, &waits);
-    while (count > 0) {
+
+    while( (newcnt > 0) || (card->outList->spoutcount)){
         set_current_state(TASK_INTERRUPTIBLE);
+        
+        if(!card->digitalstate) {
+            if (!ret) ret = -EFAULT;
+            goto writedone;
+        }
+        
         spin_lock_irqsave(&state->lock, flags);
         if (PM_SUSPENDED(card)) {
             spin_unlock_irqrestore(&state->lock, flags);
@@ -395,8 +420,17 @@ ssize_t Nvaudio_spdifwrite(struct Nvaudio_card *card, const char *buffer, size_t
 #ifdef NV_DEBUG_SP
         printk(KERN_INFO "Nvaudio_spwrite: %d bytes available space\n", cnt);
 #endif
-        if (cnt > count)
-            cnt = count;
+        if(state->rate == SAMPLERATE_48K) {
+            /* recalculate the cnt */
+            if(card->numchannels == CHANNEL_51) {
+                cnt +=  (((cnt >> 1) / CHANNEL_51) * ( 4 * sizeof(short)));
+            }else if(card->numchannels == CHANNEL_QUAD){
+                cnt +=  (((cnt >> 1) / CHANNEL_QUAD) * ( 2 * sizeof(short)));
+            }
+
+            if (cnt > newcnt)
+                cnt = newcnt;
+        }
         /* Lop off the last two bits to force the code to always
          * write in full samples.  This keeps software that sets
          * O_NONBLOCK but doesn't check the return value of the
@@ -450,7 +484,7 @@ ssize_t Nvaudio_spdifwrite(struct Nvaudio_card *card, const char *buffer, size_t
 #endif
 
                   
-         if(card->ac3passthrough) {
+       if(card->ac3passthrough) {
             if (copy_from_user(dmabuf->rawbuf+swptr,buffer,cnt)) {
                     if (!ret) ret = -EFAULT;
                     goto writedone;
@@ -458,263 +492,59 @@ ssize_t Nvaudio_spdifwrite(struct Nvaudio_card *card, const char *buffer, size_t
             SampleCount = cnt;
         }
         else {
-            /* Apply the effect of master and pcm vol to spdif data*/
-            pSource = (short *) buffer;
-            pDest   = (short *) (dmabuf->rawbuf + swptr);
-            loopend = cnt  >> 1; /* now we supporting only 16bit Samples*/
-            SampleCount = 0;
-            StartIndex  = 0;
-            shiftdone   = 0;
-
-            switch(card->numchannels) {
-                case 2:
-                    /* taking 16bit data at a time, cnt in bytes */
-                    for(loop = 0; loop < loopend; loop += 2) {
-                        /*each vol ranges from 0 - 100 */
-                        left    = *pSource;
-                        pSource += 1;
-                        right   = *pSource;
-                        pSource += 1;
-
-                        left   = (left * mastervolleft * pcmvolleft) / 10000; 
-                        right  = (right * mastervolright * pcmvolright) / 10000; 
-
-                        *pDest = (short) left;
-                        pDest  += 1;
-                        *pDest = (short) right;
-                        pDest  += 1;
-                    
-                        SampleCount +=1;
-                    }
-                    break;
-                case 4:
-                    for(loop=0;loop < loopend;loop +=4) {
-                        left = *pSource;
-                        pSource += 1;
-                        right = *pSource;
-                        pSource += 1;
-                        sleft = *pSource;
-                        pSource += 1;
-                        sright = *pSource;
-                        pSource += 1;
-
-                        left = left + ((sleft * 0xB4FD) >>16);      //left + .707*surround left
-                        right = right + ((sright * 0xB4FD) >>16);    //right + .707*surround right
-
-                        if(left>32767)
-                            left = 32767;
-                        else if(left<-32768)
-                            left = -32768;
-
-                        if(right>32767)
-                            right = 32767;
-                        else if(right<-32768)
-                            right = -32768;
-
-                        /* apply the volume */
-                        left   = ( left * mastervolleft * pcmvolleft) / 10000; 
-                        right  = ( right * mastervolright * pcmvolright) / 10000; 
-
-                        *pDest = (short)left;
-                        pDest += 1;
-                        *pDest = (short)right;
-                        pDest += 1;
-
-                        SampleCount += 1;
-                    }
-
-                    break;
-                case 6:
+            if(state->rate == SAMPLERATE_48K) {
+                loopend = cnt  >> 1; /* now we supporting only 16bit Samples*/
+                SampleCount = 0;
             
-                    if(SPDIFLeftoverCount > 0) {
-                
-                        left = SPDIFLeftovers[0];
+                switch(card->numchannels) {
+                    case CHANNEL_STEREO:
+                        /* taking 16bit data at a time, cnt in bytes */
+                        SampleCount = Nvaudio_MixNormal(buffer,state,loopend, CHANNEL_STEREO, TRUE);
+                        break;
 
-                        switch(SPDIFLeftoverCount)
-                        {
-                            case 1:
-                                if(loopend >= 5){
-                                right = *pSource;
-                                pSource += 1;
-                                center = *pSource;
-                                pSource += 1;
-                                lfe = *pSource;
-                                pSource += 1;
-                                sleft = *pSource;
-                                pSource += 1;
-                                sright = *pSource;
-                                pSource += 1;
-                                shiftdone = 1;
-                                }
-                                break;
+                    case CHANNEL_QUAD:
+                        SampleCount = Nvaudio_MixDownQuadToStereo(buffer,state,loopend,CHANNEL_QUAD,TRUE);
+                        break;
 
-                            case 2:
-                                if(loopend >= 4){
-                                right  = SPDIFLeftovers[1];
-                                center = *pSource;
-                                pSource += 1;
-                                lfe = *pSource;
-                                pSource += 1;
-                                sleft = *pSource;
-                                pSource +=1 ;
-                                sright = *pSource;
-                                pSource += 1;
-                                shiftdone = 1;
-                                }
-                                break;
-                            case 3:
-                                if(loopend >= 3){
-                                right  = SPDIFLeftovers[1];
-                                center = SPDIFLeftovers[2];
-                                lfe = *pSource;
-                                pSource += 1;
-                                sleft = *pSource;
-                                pSource += 1;
-                                sright = *pSource;
-                                pSource += 1;
-                                shiftdone = 1;
-                                }
-                                break;
-                            case 4:
-                                if(loopend >= 2){
-                                right  = SPDIFLeftovers[1];
-                                center = SPDIFLeftovers[2];
-                                lfe = SPDIFLeftovers[3];
-                                sleft = *pSource;
-                                pSource += 1;
-                                sright = *pSource;
-                                pSource += 1;
-                                shiftdone = 1;
-                                }
-                                break;
-                            case 5:
-                                if(loopend >= 1){
-                                right  = SPDIFLeftovers[1];
-                                center = SPDIFLeftovers[2];
-                                lfe = SPDIFLeftovers[3];
-                                sleft  = SPDIFLeftovers[4];
-                                sright = *pSource;
-                                pSource += 1;
-                                shiftdone = 1;
-                                }
-                                break;
-                            default:
-                                break;
-
-                        } // switch
-
-                        if(shiftdone == 0) {
-                            /* Copy the rest of the data and leave*/
-                            for(loop=SPDIFLeftoverCount; loop <(SPDIFLeftoverCount+loopend); loop++) {
-                                SPDIFLeftovers[loop] = *pSource;
-                                pSource += 1;
-                            }
-                            SPDIFLeftoverCount += loopend;
-                            break;
-                        }
-
-                        left  = left + center + ((sleft * 0xB4FD) >>16) + (lfe / 2);     //left + .707*surround left + 0.5*LFE
-                        right = right + center + ((sright * 0xB4FD) >>16) + (lfe / 2);  //right +.707*surround right + 0.5*LFE
-
-                        if(left>32767)
-                            left = 32767;
-                        else if(left<-32768)
-                            left = -32768;
-
-                        if(right>32767)
-                            right = 32767;
-                        else if(right<-32768)
-                            right = -32768;
-
-
-                        /* apply the volume */
-                        left   = ( left * mastervolleft * pcmvolleft) / 10000; 
-                        right  = ( right * mastervolright * pcmvolright) / 10000; 
-
-                        *pDest = (short)left;
-                        pDest += 1;
-                        *pDest = (short)right;
-                        pDest += 1;
-               
-                        StartIndex = 6 - SPDIFLeftoverCount;
-                        SampleCount = 1;
-                    } // if
-
-
-                    for(loop=StartIndex;loop<=(loopend-6); loop+=6)  {
-                        left = *pSource;
-                        pSource += 1;
-                        right = *pSource;
-                        pSource += 1;
-                        center = *pSource;
-                        pSource += 1;
-                        lfe = *pSource;
-                        pSource += 1;
-                        sleft = *pSource;
-                        pSource += 1;
-                        sright = *pSource;
-                        pSource += 1;
-
-                        left  = left + center + ((sleft * 0xB4FD) >>16) + (lfe / 2);    //left + .707*surround left + 0.5*LFE
-                        right = right + center + ((sright * 0xB4FD) >>16) + (lfe / 2);  //right +.707*surround right + 0.5*LFE
-
-
-                        if(left>32767)
-                            left = 32767;
-                        else if(left<-32768)
-                            left = -32768;
-
-                        if(right>32767)
-                            right = 32767;
-                        else if(right<-32768)
-                            right = -32768;
-
-                        /* apply the volume */
-                        left   = ( left * mastervolleft * pcmvolleft) / 10000; 
-                        right  = ( right * mastervolright * pcmvolright) / 10000; 
-
-                        *pDest = (short)left;
-                        pDest += 1;
-                        *pDest = (short)right;
-                        pDest += 1;
-
-                        SampleCount += 1;
-
-                    }
-            
-                    //Check for leftovers
-                    if(loop != loopend) {
-                        SPDIFLeftoverCount = loopend - loop;
-                        for(loop=0; loop < SPDIFLeftoverCount; loop++) {
-                            SPDIFLeftovers[loop] = *pSource;
-                            pSource += 1;
-                        }
-                    } else  {
-                        SPDIFLeftoverCount = 0;
-                    }
-                    break;
-            } //switch for channel num
+                    case CHANNEL_51:
+                        SampleCount = Nvaudio_MixDown5_1ToStereo(buffer,state,loopend,CHANNEL_51,TRUE);
+                        break;
+                } //switch for channel num
         
-            SampleCount = (SampleCount << 2); /* 16bit stereo to get in Bytes*/
+                SampleCount = (SampleCount << 2); /* 16bit stereo to get in Bytes*/
+
+            }else {
+                
+                newcnt = count;
+                if((newcnt > 0) && (cnt < newcnt)) {
+                        newcnt = cnt;
+                }
+                newcnt = Nvaudio_copydatatospprd(card,newcnt);
+                swptr  = dmabuf->swptr;
+            }
 
         } //else
 
-        swptr = (swptr + SampleCount) % dmabuf->dmasize;
+        if( (card->ac3passthrough)  || 
+            (state->rate == 48000)) {
+        
+            swptr = (swptr + SampleCount) % dmabuf->dmasize;
 
-        spin_lock_irqsave(&state->lock, flags);
-        if (PM_SUSPENDED(card)) {
-                spin_unlock_irqrestore(&state->lock, flags);
-                continue;
+            spin_lock_irqsave(&state->lock, flags);
+            if (PM_SUSPENDED(card)) {
+                    spin_unlock_irqrestore(&state->lock, flags);
+                    continue;
+            }
+
+            dmabuf->swptr = swptr;
+            dmabuf->count += SampleCount;
+            spin_unlock_irqrestore(&state->lock, flags);
+
+            newcnt -= cnt;
+            buffer += cnt;
+            ret    += cnt;
         }
 
-        dmabuf->swptr = swptr;
-        dmabuf->count += SampleCount;
-        spin_unlock_irqrestore(&state->lock, flags);
-
-        count  -= cnt;
-        buffer += cnt;
-        ret    += cnt;
-
 #ifdef NV_DEBUG_SP
         printk("spwrite swptr %x dmacount %x \n", swptr,dmabuf->count);
 #endif
@@ -728,9 +558,16 @@ writedone:
     Nvaudio_update_splvi(state);
     set_current_state(TASK_RUNNING);
     remove_wait_queue(&dmabuf->wait, &waits);
+    if((state->rate != 48000) && (!card->ac3passthrough)){
+        buffer += count;
+        ret    += count;
+    }
     return ret;
 }
 
+/**********************************************************************
+* Nvaudio_channel_spdifinterrupt
+**********************************************************************/
 void Nvaudio_channel_spdifinterrupt(struct Nvaudio_card *card)
 {
 #ifdef DEBUG_INTERRUPTS
