@@ -8,7 +8,7 @@
 #ifndef _BASETYPE_H_
 #define _BASETYPE_H_
 
-#define HDB_VERSION_STRING "HDR B: $Revision: #4 $"
+#define HDB_VERSION_STRING "HDR B: $Revision: #5 $"
 
 // Fundamental data types
 // UCHAR    8 bit unsigned
@@ -19,7 +19,12 @@
 typedef unsigned char   UCHAR;
 typedef unsigned int    UINT;
 typedef unsigned short  USHORT;
+#ifndef linux
 typedef unsigned long   ULONG;
+#else
+typedef unsigned int    ULONG;
+#endif
+
 #define VOID            void
 
 // Constructed types
