@@ -20,7 +20,7 @@
 #include "adapter.h"
 #include "nvnet.h"
 
-char *linux_src_version_string = "NVNET Source $Revision: #13 $";
+char *linux_src_version_string = "NVNET Source $Revision: #16 $";
 char *common_hdA_version_string = HDA_VERSION_STRING;
 char *common_hdB_version_string = HDB_VERSION_STRING;
 char *common_hdO_version_string = HDO_VERSION_STRING;
@@ -31,8 +31,8 @@ char *common_hdP_version_string = HDP_VERSION_STRING;
  */ 
 
 #define DRIVER_NAME           "nvnet"
-#define DRIVER_VERSION        "1.0.5"
-#define DRIVER_RELEASE_DATE   "March 24, 2003"
+#define DRIVER_VERSION        "1.0.6"
+#define DRIVER_RELEASE_DATE   "June 23, 2003"
 
 MODULE_AUTHOR("NVIDIA (linux-nforce-bugs@nvidia.com");
 MODULE_DESCRIPTION("NVIDIA Corporation NVNET Ethernet Driver Version"
@@ -144,7 +144,6 @@ static int linuxalloc(void *ctx, MEMORY_BLOCK *mem)
 {
     struct net_device *dev = (struct net_device *)ctx;
     struct nvnet_private *priv  = dev->priv;
-    u32 mem_physical;
     dma_addr_t dma_addr;
 
     PRINTK(DEBUG_INIT, "linuxalloc: In\n");
@@ -158,8 +157,7 @@ static int linuxalloc(void *ctx, MEMORY_BLOCK *mem)
     }
 
     memset(mem->pLogical, 0, mem->uiLength);
-    mem_physical   = (u32)dma_addr;
-    mem->pPhysical = (PVOID)mem_physical;
+    mem->pPhysical = (PVOID)dma_addr;
 
     PRINTK(DEBUG_INIT,"linuxalloc: Out\n");
     return 1;
@@ -173,9 +171,7 @@ static int linuxfree(void *ctx, MEMORY_BLOCK *mem)
 {
     struct net_device *dev    = (struct net_device *)ctx;
     struct nvnet_private *priv  = dev->priv;
-
-    u32 mem_physical    = (u32)mem->pPhysical;
-    dma_addr_t dma_addr = (dma_addr_t)mem_physical;
+    dma_addr_t dma_addr = (dma_addr_t)mem->pPhysical;
  
     PRINTK(DEBUG_INIT, "linuxfree: In\n");
 
@@ -212,7 +208,7 @@ static int linuxallocex(void *ctx, MEMORY_BLOCKEX *mem_block_ex)
     }
 
     mem_block_ex->pLogicalOrig = mem_block.pLogical;
-    mem_block_ex->pPhysicalOrigLow = (ULONG)mem_block.pPhysical;
+    mem_block_ex->pPhysicalOrigLow = (unsigned long)mem_block.pPhysical;
     mem_block_ex->pPhysicalOrigHigh = 0;
 
     mem_block_ex->pPhysical = mem_block.pPhysical;
@@ -225,9 +221,9 @@ static int linuxallocex(void *ctx, MEMORY_BLOCKEX *mem_block_ex)
 
         if (offset)
         {
-            mem_block_ex->pPhysical = (PVOID)((ULONG)mem_block_ex->pPhysical + 
+            mem_block_ex->pPhysical = (PVOID)((unsigned long)mem_block_ex->pPhysical + 
                                               mem_block_ex->AlignmentSize - offset);
-            mem_block_ex->pLogical = (PVOID)((ULONG)mem_block_ex->pLogical +
+            mem_block_ex->pLogical = (PVOID)((unsigned long)mem_block_ex->pLogical +
                                             mem_block_ex->AlignmentSize - offset);
         }   // if (offset)
     }       // if (mem_block_ex->uiLength != mem_block_ex->uiLengthOrig)
@@ -243,7 +239,7 @@ static int linuxfreeex(void *ctx, MEMORY_BLOCKEX *mem_block_ex)
     MEMORY_BLOCK mem_block;
 
     mem_block.pLogical = mem_block_ex->pLogicalOrig;
-    mem_block.pPhysical = (PVOID)mem_block_ex->pPhysicalOrigLow;
+    mem_block.pPhysical = (PVOID)((unsigned long)mem_block_ex->pPhysicalOrigLow);
     mem_block.uiLength = mem_block_ex->uiLengthOrig;
 
     return linuxfree(ctx, &mem_block);
@@ -261,7 +257,7 @@ static int linuxclear(void *ctx, void *mem, int length)
 /*
  * Maps to pfnStallExecution
  */
-static int linuxdelay(void *ctx, unsigned long usec)
+static int linuxdelay(void *ctx, ULONG usec)
 {
     udelay(usec);
     return 1;
@@ -293,11 +289,11 @@ static int linuxallocrxbuf(void *ctx, MEMORY_BLOCK *mem, void **id)
     mem->uiLength = MAX_PACKET_SIZE;              
     *id           = (void *)info;        
 
-    //skb_reserve(skb, 2);        /* Get 16 byte alignment of IP part */
+    skb_reserve(skb, 2);        /* Get 16 byte alignment of IP part */
 
     info->dma  = pci_map_single(priv->pdev, skb->tail, \
                        MAX_PACKET_SIZE, PCI_DMA_FROMDEVICE);
-    mem->pPhysical = (void *)cpu_to_le32(info->dma);
+    mem->pPhysical = (void *)((unsigned long)cpu_to_le32(info->dma));
     return 1;
 
 }
@@ -334,7 +330,7 @@ static int linuxfreerxbuf(void *ctx, MEMORY_BLOCK *mem, void *id)
  * Maps to pfnPacketWasSent
  */
 
-static int linuxpackettx(void *ctx, void *id, unsigned long success)
+static int linuxpackettx(void *ctx, void *id, ULONG success)
 {
     struct net_device *dev = (struct net_device *)ctx;
     struct nvnet_private *priv = (struct nvnet_private *)dev->priv;
@@ -372,7 +368,7 @@ static int linuxpackettx(void *ctx, void *id, unsigned long success)
 /*
  * Maps to pfnPacketWasReceived
  */
-static int linuxpacketrx(void *ctx, void *data, unsigned long success, 
+static int linuxpacketrx(void *ctx, void *data, ULONG success, 
 unsigned char *newbuf, unsigned char priority)
 {
     struct sk_buff *skb;
@@ -510,6 +506,7 @@ static void nvnet_multicast(struct net_device *dev)
     u8 oraddr[6];
     u8 andaddr[6];
     int i;
+    unsigned long lockflags;
 
     PRINTK(DEBUG_OPEN, "nvnet_multicast: In\n");
 
@@ -526,7 +523,7 @@ static void nvnet_multicast(struct net_device *dev)
         hwfilter.acMulticastMask[i]     = 0;
     }
 
-    spin_lock_irqsave(&priv->lock, priv->lockflags);
+    spin_lock_irqsave(&priv->lock, lockflags);
 
     if(dev->flags & IFF_PROMISC)
     {
@@ -592,7 +589,7 @@ static void nvnet_multicast(struct net_device *dev)
     } /* else */
 
     hw->pfnSetPacketFilter(hw->pADCX, &hwfilter);
-    spin_unlock_irqrestore(&priv->lock, priv->lockflags);
+    spin_unlock_irqrestore(&priv->lock, lockflags);
     PRINTK(DEBUG_OPEN, "nvnet_multicast: Out\n");
 
 }
@@ -606,7 +603,6 @@ static int nvnet_xmit(struct sk_buff *skb, struct net_device *dev)
     ADAPTER_WRITE_DATA txdata;
     ADAPTER_API *hw;
     int status;
-    dma_addr_t addr;
     int retval = 0;
     nvnet_tx_info_os_t *tx_ring;
     unsigned long lockflags;
@@ -654,12 +650,13 @@ static int nvnet_xmit(struct sk_buff *skb, struct net_device *dev)
     tx_ring->pvID = (PVOID)skb;
     tx_ring->pCoalesceBuffer = NULL;
 
-    addr  = cpu_to_le32(
-               pci_map_single(priv->pdev, skb->data, skb->len, \
-               PCI_DMA_TODEVICE));
-
-    tx_ring->dma_addr[0] = addr;
-    txdata.sElement[0].pPhysical = (void *)((u32)addr);
+    tx_ring->dma_addr[0] = pci_map_single(
+				priv->pdev, 
+			        skb->data, 
+                                skb->len, 
+                                PCI_DMA_TODEVICE);
+    txdata.sElement[0].pPhysical = (void *)((unsigned long)
+                        cpu_to_le32(tx_ring->dma_addr[0]));
     txdata.ulTotalLength = txdata.sElement[0].ulLength = skb->len;
 
     status = hw->pfnWrite(hw->pADCX, &txdata);
@@ -685,7 +682,7 @@ static int nvnet_xmit(struct sk_buff *skb, struct net_device *dev)
             break;
     }
 
-    spin_unlock_irqrestore(&priv->lock, priv->lockflags);
+    spin_unlock_irqrestore(&priv->lock, lockflags);
     PRINTK(DEBUG_TX, "nvnet_xmit - Out\n");
     return retval;
 
@@ -860,7 +857,7 @@ static int nvnet_ioctl(struct net_device *dev, struct ifreq *request, int cmd)
         case SIOCDEVPRIVATE+1:        /* for binary compat, remove in 2.5 */
 
             if(pass != ADAPTER_ReadPhy(hw->pADCX, priv->phyaddr, \
-                                       data[1] & 0x1f, (ulong *)&data[3])) 
+                                       data[1] & 0x1f, (ULONG *)&data[3])) 
             {
                 PRINTK_ERROR("nvnet_ioctl - ReadPhy failed\n");
                 retval = -EBUSY;
@@ -895,7 +892,7 @@ static int nvnet_init(struct net_device *dev)
     int status;
     struct nvnet_private *priv;
     u32 i;
-    ADAPTER_OPEN_PARAMS        OpenParams;
+    ADAPTER_OPEN_PARAMS        OpenParams = { 0 };
 
     PRINTK(DEBUG_INIT, "nvnet_init - In\n");
 
@@ -944,9 +941,9 @@ static int nvnet_init(struct net_device *dev)
     priv->linkup    = true;
 
     PRINTK(DEBUG_INIT, "nvnet_init - call adapter open\n");
-    OpenParams.MaxDpcLoop = 2;
-    OpenParams.MaxRxPkt = 64;
-    OpenParams.MaxTxPkt = 64;
+    OpenParams.MaxDpcLoop = 1;
+    OpenParams.MaxRxPkt = 16;
+    OpenParams.MaxTxPkt = 16;
     OpenParams.SentPacketStatusSuccess = 1;
     OpenParams.SentPacketStatusFailure = 0;
     OpenParams.MaxRxPktToAccumulate = 6;
